% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/response_gen.R
\name{response_gen}
\alias{response_gen}
\title{Generation of item response data using a rotated block design}
\usage{
response_gen(subject, item, theta, a_par = NULL, b_par, c_par = NULL,
  d_par = NULL, item_no = NULL, ogive = "Logistic")
}
\arguments{
\item{subject}{integer vector of test taker IDs.}

\item{item}{integer vector of item IDs.}

\item{theta}{numeric vector of latent test taker abilities.}

\item{a_par}{numeric vector of item a parameters for each item.}

\item{b_par}{numeric vector of item b parameters for each item.}

\item{c_par}{numeric vector of item c parameters for each item.}

\item{d_par}{list of numeric vectors of item threshold parameters for each item.}

\item{item_no}{vector of item numbers the correspond the item parameters}

\item{ogive}{can be "Normal" or "Logistic".}
}
\description{
Creates a data frame of discrete item responses based on.
}
\section{Details}{

\code{subject} and \code{item} must be equal lengths.  

Generalized partial credit models (\code{!is.null(d_par)}) uses threshold parameterization.
}

\examples{
set.seed(1234)
s_id <- c(1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 4, 4, 
          4, 4, 4, 4, 4, 5, 5, 5, 5, 5, 5, 5, 6, 6, 6, 6, 6, 6, 7, 7, 7, 7, 7, 
          7, 7, 8, 8, 8, 8, 8, 8, 9, 9, 9, 9, 9, 9, 9, 10, 10, 10, 10, 10, 10, 
          10, 11, 11, 11, 11, 11, 11, 12,12, 12, 12, 12, 12, 12, 13, 13, 13, 13, 
          13, 13, 14, 14, 14, 14, 14, 14, 15, 15, 15, 15, 15, 15, 16,16, 16, 16, 
          16, 16, 17, 17, 17, 17, 17, 17, 17, 18, 18, 18, 18, 18, 18, 18, 19, 19, 
          19, 19, 19, 19,19, 20, 20, 20, 20, 20, 20, 20)
i_id<- c(1, 4, 7, 10, 3, 6, 9, 1, 4, 7, 10, 2, 5, 8, 1, 4, 7, 10, 3, 6, 9, 1, 4, 
         7, 10, 3, 6, 9, 1, 4, 7, 10, 3, 6, 9, 2, 5, 8, 3, 6, 9, 1, 4, 7, 10, 2, 
         5, 8, 2, 5, 8, 3, 6, 9, 1, 4, 7, 10, 2, 5, 8, 1, 4, 7, 10, 3, 6, 9, 2, 
         5, 8, 3, 6, 9, 1, 4, 7, 10, 3, 6, 9, 2, 5, 8, 3, 6, 9, 2, 5, 8, 3, 6, 9, 
         2, 5, 8, 3, 6, 9, 2, 5, 8, 3, 6, 9, 1, 4, 7, 10, 2, 5, 8, 1, 4, 7, 10, 
         2, 5, 8, 1, 4, 7, 10, 2, 5, 8, 1, 4, 7, 10, 3, 6, 9)
bb <- c(-1.72, -1.85, 0.98, 0.07, 1.00, 0.13, -0.43, -0.29, 0.86, 1.26)
aa <- c(1.28, 0.78, 0.98, 1.21, 0.83, 1.01, 0.92, 0.76, 0.88, 1.11)
cc <- rep(0, 10)
dd <- list(c(0, 0, -0.13, 0, -0.19, 0, 0, 0, 0, 0), 
           c(0, 0,  0.13, 0,  0.19, 0, 0, 0, 0, 0))
response_gen(subject = s_id, item = i_id, theta = rnorm(20, 0, 1), 
             b_par = bb, a_par = aa, c_par = cc, d_par = dd)

}
